@@ -208,7 +208,7 @@ public class RequestFacade implements HttpServletRequest {
     /**
      * The wrapped request.
      */
-    protected Request request = null;
+    protected Request request;
 
 
     /**
@@ -642,7 +642,6 @@ public class RequestFacade implements HttpServletRequest {
 
     @Override
     public HttpSession getSession() {
-        checkFacade();
         return getSession(true);
     }
 
@@ -797,7 +796,7 @@ public class RequestFacade implements HttpServletRequest {
 
     @Override
     public <T extends HttpUpgradeHandler> T upgrade(Class<T> httpUpgradeHandlerClass)
-            throws java.io.IOException, ServletException {
+            throws IOException, ServletException {
         checkFacade();
         return request.upgrade(httpUpgradeHandlerClass);
     }
