@@ -23,6 +23,7 @@ import java.util.Collections;
 import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.HashSet;
+import java.util.Objects;
 import java.util.Set;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.locks.ReentrantReadWriteLock;
@@ -101,7 +102,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     protected long available = 0L;
 
     /**
-     * The broadcaster that sends j2ee notifications.
+     * The broadcaster that sends EE notifications.
      */
     protected final NotificationBroadcasterSupport broadcaster;
 
@@ -200,7 +201,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     protected boolean swallowOutput = false;
 
     // To support jmx attributes
-    protected StandardWrapperValve swValve;
+    StandardWrapperValve swValve;
     protected long loadTime = 0;
     protected int classLoadTime = 0;
 
@@ -245,26 +246,12 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
         this.overridable = overridable;
     }
 
-    /**
-     * Return the available date/time for this servlet, in milliseconds since the epoch. If this date/time is
-     * Long.MAX_VALUE, it is considered to mean that unavailability is permanent and any request for this servlet will
-     * return an SC_NOT_FOUND error. If this date/time is in the future, any request for this servlet will return an
-     * SC_SERVICE_UNAVAILABLE error. If it is zero, the servlet is currently available.
-     */
     @Override
     public long getAvailable() {
         return this.available;
     }
 
 
-    /**
-     * Set the available date/time for this servlet, in milliseconds since the epoch. If this date/time is
-     * Long.MAX_VALUE, it is considered to mean that unavailability is permanent and any request for this servlet will
-     * return an SC_NOT_FOUND error. If this date/time is in the future, any request for this servlet will return an
-     * SC_SERVICE_UNAVAILABLE error.
-     *
-     * @param available The new available date/time
-     */
     @Override
     public void setAvailable(long available) {
         long oldAvailable = this.available;
@@ -285,9 +272,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * @return the load-on-startup order value (negative value means load on first call).
-     */
     @Override
     public int getLoadOnStartup() {
 
@@ -303,11 +287,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * Set the load-on-startup order value (negative value means load on first call).
-     *
-     * @param value New load-on-startup value
-     */
     @Override
     public void setLoadOnStartup(int value) {
 
@@ -363,20 +342,12 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * @return the run-as identity for this servlet.
-     */
     @Override
     public String getRunAs() {
         return this.runAs;
     }
 
 
-    /**
-     * Set the run-as identity for this servlet.
-     *
-     * @param runAs New run-as identity value
-     */
     @Override
     public void setRunAs(String runAs) {
 
@@ -387,20 +358,12 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * @return the fully qualified servlet class name for this servlet.
-     */
     @Override
     public String getServletClass() {
         return this.servletClass;
     }
 
 
-    /**
-     * Set the fully qualified servlet class name for this servlet.
-     *
-     * @param servletClass Servlet class name
-     */
     @Override
     public void setServletClass(String servletClass) {
 
@@ -426,9 +389,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * @return <code>true</code> if the Servlet has been marked unavailable.
-     */
     @Override
     public boolean isUnavailable() {
 
@@ -470,15 +430,20 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
             for (int i = 0; methods != null && i < methods.length; i++) {
                 Method m = methods[i];
 
-                if (m.getName().equals("doGet")) {
-                    allow.add("GET");
-                    allow.add("HEAD");
-                } else if (m.getName().equals("doPost")) {
-                    allow.add("POST");
-                } else if (m.getName().equals("doPut")) {
-                    allow.add("PUT");
-                } else if (m.getName().equals("doDelete")) {
-                    allow.add("DELETE");
+                switch (m.getName()) {
+                    case "doGet":
+                        allow.add("GET");
+                        allow.add("HEAD");
+                        break;
+                    case "doPost":
+                        allow.add("POST");
+                        break;
+                    case "doPut":
+                        allow.add("PUT");
+                        break;
+                    case "doDelete":
+                        allow.add("DELETE");
+                        break;
                 }
             }
         }
@@ -487,18 +452,12 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * @return the associated servlet instance.
-     */
     @Override
     public Servlet getServlet() {
         return instance;
     }
 
 
-    /**
-     * Set the associated servlet instance.
-     */
     @Override
     public void setServlet(Servlet servlet) {
         instance = servlet;
@@ -507,10 +466,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
 
     // --------------------------------------------------------- Public Methods
 
-    /**
-     * Execute a periodic task, such as reloading, etc. This method will be invoked inside the classloading context of
-     * this container. Unexpected throwables will be caught and logged.
-     */
     @Override
     public synchronized void backgroundProcess() {
         super.backgroundProcess();
@@ -534,7 +489,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
      */
     public static Throwable getRootCause(ServletException e) {
         Throwable rootCause = e;
-        Throwable rootCauseCheck = null;
+        Throwable rootCauseCheck;
         // Extra aggressive rootCause finding
         int loops = 0;
         do {
@@ -561,12 +516,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * Add a new servlet initialization parameter for this servlet.
-     *
-     * @param name  Name of this initialization parameter to add
-     * @param value Value of this initialization parameter to add
-     */
     @Override
     public void addInitParameter(String name, String value) {
 
@@ -581,11 +530,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * Add a mapping associated with the Wrapper.
-     *
-     * @param mapping The new wrapper mapping
-     */
     @Override
     public void addMapping(String mapping) {
 
@@ -602,12 +546,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * Add a new security role reference record to the set of records for this servlet.
-     *
-     * @param name Role name used within this servlet
-     * @param link Role name used within the web application
-     */
     @Override
     public void addSecurityReference(String name, String link) {
 
@@ -622,12 +560,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * Allocate an initialized instance of this Servlet that is ready to have its <code>service()</code> method called.
-     *
-     * @exception ServletException if the servlet init() method threw an exception
-     * @exception ServletException if a loading error occurs
-     */
     @Override
     public Servlet allocate() throws ServletException {
 
@@ -643,8 +575,8 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
             synchronized (this) {
                 if (instance == null) {
                     try {
-                        if (log.isDebugEnabled()) {
-                            log.debug("Allocating instance");
+                        if (log.isTraceEnabled()) {
+                            log.trace("Allocating instance");
                         }
                         instance = loadServlet();
                         newInstance = true;
@@ -653,9 +585,9 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
                         countAllocated.incrementAndGet();
                     } catch (ServletException e) {
                         throw e;
-                    } catch (Throwable e) {
-                        ExceptionUtils.handleThrowable(e);
-                        throw new ServletException(sm.getString("standardWrapper.allocate"), e);
+                    } catch (Throwable t) {
+                        ExceptionUtils.handleThrowable(t);
+                        throw new ServletException(sm.getString("standardWrapper.allocate"), t);
                     }
                 }
                 if (!instanceInitialized) {
@@ -676,24 +608,12 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * Decrement the allocation count for this servlet.
-     *
-     * @param servlet The servlet to be returned
-     *
-     * @exception ServletException if a deallocation error occurs
-     */
     @Override
     public void deallocate(Servlet servlet) throws ServletException {
         countAllocated.decrementAndGet();
     }
 
 
-    /**
-     * Return the value for the specified initialization parameter name, if any; otherwise return <code>null</code>.
-     *
-     * @param name Name of the requested initialization parameter
-     */
     @Override
     public String findInitParameter(String name) {
 
@@ -707,9 +627,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * Return the names of all defined initialization parameters for this servlet.
-     */
     @Override
     public String[] findInitParameters() {
 
@@ -723,9 +640,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * Return the mappings associated with this wrapper.
-     */
     @Override
     public String[] findMappings() {
 
@@ -739,15 +653,9 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * Return the security role link for the specified security role reference name, if any; otherwise return
-     * <code>null</code>.
-     *
-     * @param name Security role reference used within this servlet
-     */
     @Override
     public String findSecurityReference(String name) {
-        String reference = null;
+        String reference;
 
         referencesLock.readLock().lock();
         try {
@@ -770,10 +678,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * Return the set of security role reference names associated with this servlet, if any; otherwise return a
-     * zero-length array.
-     */
     @Override
     public String[] findSecurityReferences() {
 
@@ -788,17 +692,12 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
 
 
     /**
-     * Load and initialize an instance of this servlet, if there is not already at least one initialized instance. This
-     * can be used, for example, to load servlets that are marked in the deployment descriptor to be loaded at server
-     * startup time.
+     * {@inheritDoc}
      * <p>
      * <b>IMPLEMENTATION NOTE</b>: Servlets whose classnames begin with <code>org.apache.catalina.</code> (so-called
      * "container" servlets) are loaded by the same classloader that loaded this class, rather than the classloader for
      * the current web application. This gives such classes access to Catalina internals, which are prevented for
      * classes loaded for web applications.
-     *
-     * @exception ServletException if the servlet init() method threw an exception
-     * @exception ServletException if some other loading problem occurs
      */
     @Override
     public synchronized void load() throws ServletException {
@@ -822,9 +721,9 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
 
             try {
                 jspMonitorON = new ObjectName(oname.toString());
-                Registry.getRegistry(null, null).registerComponent(instance, jspMonitorON, null);
-            } catch (Exception ex) {
-                log.warn(sm.getString("standardWrapper.jspMonitorError", instance));
+                Registry.getRegistry(null).registerComponent(instance, jspMonitorON, null);
+            } catch (Exception e) {
+                log.warn(sm.getString("standardWrapper.jspMonitorError", instance), e);
             }
         }
     }
@@ -867,19 +766,19 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
                 unavailable(null);
                 // Restore the context ClassLoader
                 throw new ServletException(sm.getString("standardWrapper.notServlet", servletClass), e);
-            } catch (Throwable e) {
-                e = ExceptionUtils.unwrapInvocationTargetException(e);
-                ExceptionUtils.handleThrowable(e);
+            } catch (Throwable t) {
+                Throwable throwable = ExceptionUtils.unwrapInvocationTargetException(t);
+                ExceptionUtils.handleThrowable(throwable);
                 unavailable(null);
 
                 // Added extra log statement for Bugzilla 36630:
                 // https://bz.apache.org/bugzilla/show_bug.cgi?id=36630
                 if (log.isDebugEnabled()) {
-                    log.debug(sm.getString("standardWrapper.instantiate", servletClass), e);
+                    log.debug(sm.getString("standardWrapper.instantiate", servletClass), throwable);
                 }
 
                 // Restore the context ClassLoader
-                throw new ServletException(sm.getString("standardWrapper.instantiate", servletClass), e);
+                throw new ServletException(sm.getString("standardWrapper.instantiate", servletClass), throwable);
             }
 
             if (multipartConfigElement == null) {
@@ -906,7 +805,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
         } finally {
             if (swallowOutput) {
                 String log = SystemLogHandler.stopCapture();
-                if (log != null && log.length() > 0) {
+                if (log != null && !log.isEmpty()) {
                     if (getServletContext() != null) {
                         getServletContext().log(log);
                     } else {
@@ -961,11 +860,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
         }
     }
 
-    /**
-     * Remove the specified initialization parameter from this servlet.
-     *
-     * @param name Name of the initialization parameter to remove
-     */
     @Override
     public void removeInitParameter(String name) {
 
@@ -980,11 +874,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * Remove a mapping associated with the wrapper.
-     *
-     * @param mapping The pattern to remove
-     */
     @Override
     public void removeMapping(String mapping) {
 
@@ -1001,11 +890,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * Remove any security role reference for the specified role name.
-     *
-     * @param name Security role used within this servlet to be removed
-     */
     @Override
     public void removeSecurityReference(String name) {
 
@@ -1020,12 +904,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * Process an UnavailableException, marking this servlet as unavailable for the specified amount of time.
-     *
-     * @param unavailable The exception that occurred, or <code>null</code> to mark this servlet as permanently
-     *                        unavailable
-     */
     @Override
     public void unavailable(UnavailableException unavailable) {
         getServletContext().log(sm.getString("standardWrapper.unavailable", getName()));
@@ -1044,13 +922,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * Unload all initialized instances of this servlet, after calling the <code>destroy()</code> method for each
-     * instance. This can be used, for example, prior to shutting down the entire servlet engine, or prior to reloading
-     * all of the classes from the Loader associated with our Loader's repository.
-     *
-     * @exception ServletException if an exception is thrown by the destroy() method
-     */
     @Override
     public synchronized void unload() throws ServletException {
 
@@ -1096,11 +967,11 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
                 }
 
             } catch (Throwable t) {
-                t = ExceptionUtils.unwrapInvocationTargetException(t);
-                ExceptionUtils.handleThrowable(t);
+                Throwable throwable = ExceptionUtils.unwrapInvocationTargetException(t);
+                ExceptionUtils.handleThrowable(throwable);
                 fireContainerEvent("unload", this);
                 unloading = false;
-                throw new ServletException(sm.getString("standardWrapper.destroyException", getName()), t);
+                throw new ServletException(sm.getString("standardWrapper.destroyException", getName()), throwable);
             } finally {
                 // Annotation processing
                 if (!((Context) getParent()).getIgnoreAnnotations()) {
@@ -1114,7 +985,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
                 // Write captured output
                 if (swallowOutput) {
                     String log = SystemLogHandler.stopCapture();
-                    if (log != null && log.length() > 0) {
+                    if (log != null && !log.isEmpty()) {
                         if (getServletContext() != null) {
                             getServletContext().log(log);
                         } else {
@@ -1131,7 +1002,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
         instance = null;
 
         if (isJspServlet && jspMonitorON != null) {
-            Registry.getRegistry(null, null).unregisterComponent(jspMonitorON);
+            Registry.getRegistry(null).unregisterComponent(jspMonitorON);
         }
 
         unloading = false;
@@ -1142,21 +1013,12 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     // -------------------------------------------------- ServletConfig Methods
 
 
-    /**
-     * @return the initialization parameter value for the specified name, if any; otherwise return <code>null</code>.
-     *
-     * @param name Name of the initialization parameter to retrieve
-     */
     @Override
     public String getInitParameter(String name) {
         return findInitParameter(name);
     }
 
 
-    /**
-     * @return the set of initialization parameter names defined for this servlet. If none are defined, an empty
-     *             Enumeration is returned.
-     */
     @Override
     public Enumeration<String> getInitParameterNames() {
 
@@ -1170,9 +1032,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * @return the servlet context with which this servlet is associated.
-     */
     @Override
     public ServletContext getServletContext() {
         if (parent == null) {
@@ -1185,9 +1044,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * @return the name of this servlet.
-     */
     @Override
     public String getServletName() {
         return getName();
@@ -1319,7 +1175,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
      *                                   used
      */
     @Override
-    protected synchronized void startInternal() throws LifecycleException {
+    protected void startInternal() throws LifecycleException {
 
         // Send j2ee.state.starting notification
         if (this.getObjectName() != null) {
@@ -1349,7 +1205,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
      *                                   used
      */
     @Override
-    protected synchronized void stopInternal() throws LifecycleException {
+    protected void stopInternal() throws LifecycleException {
 
         setAvailable(Long.MAX_VALUE);
 
@@ -1373,11 +1229,11 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
         if (this.getObjectName() != null) {
             Notification notification = new Notification("j2ee.state.stopped", this.getObjectName(), sequenceNumber++);
             broadcaster.sendNotification(notification);
-        }
 
-        // Send j2ee.object.deleted notification
-        Notification notification = new Notification("j2ee.object.deleted", this.getObjectName(), sequenceNumber++);
-        broadcaster.sendNotification(notification);
+            // Send j2ee.object.deleted notification
+            notification = new Notification("j2ee.object.deleted", this.getObjectName(), sequenceNumber++);
+            broadcaster.sendNotification(notification);
+        }
 
     }
 
@@ -1407,11 +1263,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
 
         StringBuilder keyProperties = new StringBuilder(",WebModule=//");
         String hostName = getParent().getParent().getName();
-        if (hostName == null) {
-            keyProperties.append("DEFAULT");
-        } else {
-            keyProperties.append(hostName);
-        }
+        keyProperties.append(Objects.requireNonNullElse(hostName, "DEFAULT"));
 
         String contextName = getParent().getName();
         if (!contextName.startsWith("/")) {
@@ -1447,12 +1299,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * Remove a JMX notificationListener
-     *
-     * @see javax.management.NotificationEmitter#removeNotificationListener(javax.management.NotificationListener,
-     *          javax.management.NotificationFilter, java.lang.Object)
-     */
     @Override
     public void removeNotificationListener(NotificationListener listener, NotificationFilter filter, Object object)
             throws ListenerNotFoundException {
@@ -1461,15 +1307,8 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
 
     protected MBeanNotificationInfo[] notificationInfo;
 
-    /**
-     * Get JMX Broadcaster Info
-     *
-     * @see javax.management.NotificationBroadcaster#getNotificationInfo()
-     */
     @Override
     public MBeanNotificationInfo[] getNotificationInfo() {
-        // FIXME: we not send j2ee.state.failed
-        // FIXME: we not send j2ee.attribute.changed
         if (notificationInfo == null) {
             notificationInfo = new MBeanNotificationInfo[] {
                     new MBeanNotificationInfo(new String[] { "j2ee.object.created" }, Notification.class.getName(),
@@ -1489,12 +1328,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * Add a JMX-NotificationListener
-     *
-     * @see javax.management.NotificationBroadcaster#addNotificationListener(javax.management.NotificationListener,
-     *          javax.management.NotificationFilter, java.lang.Object)
-     */
     @Override
     public void addNotificationListener(NotificationListener listener, NotificationFilter filter, Object object)
             throws IllegalArgumentException {
@@ -1502,11 +1335,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
     }
 
 
-    /**
-     * Remove a JMX-NotificationListener
-     *
-     * @see javax.management.NotificationBroadcaster#removeNotificationListener(javax.management.NotificationListener)
-     */
     @Override
     public void removeNotificationListener(NotificationListener listener) throws ListenerNotFoundException {
         broadcaster.removeNotificationListener(listener);
