@@ -32,9 +32,12 @@ import org.apache.catalina.session.ManagerBase;
 import org.apache.catalina.tribes.io.ReplicationStream;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.util.collections.SynchronizedStack;
+import org.apache.tomcat.util.res.StringManager;
 
 public abstract class ClusterManagerBase extends ManagerBase implements ClusterManager {
 
+    protected static final StringManager sm = StringManager.getManager(ClusterManagerBase.class);
     private final Log log = LogFactory.getLog(ClusterManagerBase.class); // must not be static
 
     /**
@@ -50,13 +53,21 @@ public abstract class ClusterManagerBase extends ManagerBase implements ClusterM
     /**
      * cached replication valve cluster container!
      */
-    private volatile ReplicationValve replicationValve = null ;
+    private volatile ReplicationValve replicationValve = null;
 
     /**
      * send all actions of session attributes.
      */
     private boolean recordAllActions = false;
 
+    private final SynchronizedStack<DeltaRequest> deltaRequestPool = new SynchronizedStack<>();
+
+
+    protected SynchronizedStack<DeltaRequest> getDeltaRequestPool() {
+        return deltaRequestPool;
+    }
+
+
     @Override
     public CatalinaCluster getCluster() {
         return cluster;
@@ -76,37 +87,6 @@ public abstract class ClusterManagerBase extends ManagerBase implements ClusterM
         this.notifyListenersOnReplication = notifyListenersOnReplication;
     }
 
-    /**
-     * Return the string pattern used for including session attributes
-     * to replication.
-     *
-     * @return the sessionAttributeFilter
-     *
-     * @deprecated Use {@link #getSessionAttributeNameFilter()}. Will be removed
-     *             in Tomcat 8.5.x
-     */
-    @Deprecated
-    public String getSessionAttributeFilter() {
-        return getSessionAttributeNameFilter();
-    }
-
-    /**
-     * Set the pattern used for including session attributes to replication.
-     * If not set, all session attributes will be eligible for replication.
-     * <p>
-     * E.g. <code>^(userName|sessionHistory)$</code>
-     * </p>
-     *
-     * @param sessionAttributeFilter
-     *            the filter name pattern to set
-     *
-     * @deprecated Use {@link #setSessionAttributeNameFilter(String)}. Will be
-     *             removed in Tomcat 8.5.x
-     */
-    @Deprecated
-    public void setSessionAttributeFilter(String sessionAttributeFilter) {
-        setSessionAttributeNameFilter(sessionAttributeFilter);
-    }
 
     public boolean isRecordAllActions() {
         return recordAllActions;
@@ -116,20 +96,6 @@ public abstract class ClusterManagerBase extends ManagerBase implements ClusterM
         this.recordAllActions = recordAllActions;
     }
 
-    /**
-     * Check whether the given session attribute should be distributed based on
-     * attribute name only.
-     *
-     * @return true if the attribute should be distributed
-     *
-     * @deprecated Use {@link #willAttributeDistribute(String, Object)}. Will be
-     *             removed in Tomcat 8.5.x
-     */
-    @Deprecated
-    public boolean willAttributeDistribute(String name) {
-        return willAttributeDistribute(name, null);
-    }
-
 
     public static ClassLoader[] getClassLoaders(Context context) {
         ClassLoader tccl = Thread.currentThread().getContextClassLoader();
@@ -142,9 +108,9 @@ public abstract class ClusterManagerBase extends ManagerBase implements ClusterM
             classLoader = tccl;
         }
         if (classLoader == tccl) {
-            return new ClassLoader[] {classLoader};
+            return new ClassLoader[] { classLoader };
         } else {
-            return new ClassLoader[] {classLoader, tccl};
+            return new ClassLoader[] { classLoader, tccl };
         }
     }
 
@@ -155,7 +121,7 @@ public abstract class ClusterManagerBase extends ManagerBase implements ClusterM
 
     @Override
     public ReplicationStream getReplicationStream(byte[] data) throws IOException {
-        return getReplicationStream(data,0,data.length);
+        return getReplicationStream(data, 0, data.length);
     }
 
     @Override
@@ -165,11 +131,11 @@ public abstract class ClusterManagerBase extends ManagerBase implements ClusterM
     }
 
 
-    //  ---------------------------------------------------- persistence handler
+    // ---------------------------------------------------- persistence handler
 
     /**
-     * {@link org.apache.catalina.Manager} implementations that also implement
-     * {@link ClusterManager} do not support local session persistence.
+     * {@link org.apache.catalina.Manager} implementations that also implement {@link ClusterManager} do not support
+     * local session persistence.
      */
     @Override
     public void load() {
@@ -177,8 +143,8 @@ public abstract class ClusterManagerBase extends ManagerBase implements ClusterM
     }
 
     /**
-     * {@link org.apache.catalina.Manager} implementations that also implement
-     * {@link ClusterManager} do not support local session persistence.
+     * {@link org.apache.catalina.Manager} implementations that also implement {@link ClusterManager} do not support
+     * local session persistence.
      */
     @Override
     public void unload() {
@@ -211,26 +177,28 @@ public abstract class ClusterManagerBase extends ManagerBase implements ClusterM
 
     /**
      * Register cross context session at replication valve thread local
+     *
      * @param session cross context session
      */
     protected void registerSessionAtReplicationValve(DeltaSession session) {
-        if(replicationValve == null) {
-            CatalinaCluster cluster = getCluster() ;
-            if(cluster != null) {
+        if (replicationValve == null) {
+            CatalinaCluster cluster = getCluster();
+            if (cluster != null) {
                 Valve[] valves = cluster.getValves();
-                if(valves != null && valves.length > 0) {
-                    for(int i=0; replicationValve == null && i < valves.length ; i++ ){
-                        if(valves[i] instanceof ReplicationValve) replicationValve =
-                                (ReplicationValve)valves[i] ;
-                    }//for
+                if (valves != null && valves.length > 0) {
+                    for (int i = 0; replicationValve == null && i < valves.length; i++) {
+                        if (valves[i] instanceof ReplicationValve) {
+                            replicationValve = (ReplicationValve) valves[i];
+                        }
+                    } // for
 
-                    if(replicationValve == null && log.isDebugEnabled()) {
-                        log.debug("no ReplicationValve found for CrossContext Support");
-                    }//endif
-                }//end if
-            }//endif
-        }//end if
-        if(replicationValve != null) {
+                    if (replicationValve == null && log.isDebugEnabled()) {
+                        log.debug(sm.getString("clusterManager.noValve"));
+                    } // endif
+                } // end if
+            } // endif
+        } // end if
+        if (replicationValve != null) {
             replicationValve.registerReplicationSession(session);
         }
     }
@@ -241,15 +209,19 @@ public abstract class ClusterManagerBase extends ManagerBase implements ClusterM
         if (getCluster() == null) {
             Cluster cluster = getContext().getCluster();
             if (cluster instanceof CatalinaCluster) {
-                setCluster((CatalinaCluster)cluster);
+                setCluster((CatalinaCluster) cluster);
             }
         }
-        if (cluster != null) cluster.registerManager(this);
+        if (cluster != null) {
+            cluster.registerManager(this);
+        }
     }
 
     @Override
     protected void stopInternal() throws LifecycleException {
-        if (cluster != null) cluster.removeManager(this);
+        if (cluster != null) {
+            cluster.removeManager(this);
+        }
         replicationValve = null;
         super.stopInternal();
     }
