@@ -16,6 +16,8 @@
  */
 package org.apache.tomcat.util.net.openssl;
 
+import java.lang.ref.Cleaner;
+import java.lang.ref.Cleaner.Cleanable;
 import java.nio.ByteBuffer;
 import java.nio.ReadOnlyBufferException;
 import java.security.Principal;
@@ -41,7 +43,6 @@ import javax.net.ssl.SSLSessionContext;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.jni.Buffer;
-import org.apache.tomcat.jni.Library;
 import org.apache.tomcat.jni.Pool;
 import org.apache.tomcat.jni.SSL;
 import org.apache.tomcat.jni.SSLContext;
@@ -132,10 +133,8 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
 
     private static final long EMPTY_ADDR = Buffer.address(ByteBuffer.allocate(0));
 
-    // OpenSSL state
-    private final long ssl;
-    private final long networkBIO;
-    private final long aprGeneration;
+    private final OpenSSLState state;
+    private final Cleanable cleanable;
 
     private enum Accepted {
         NOT,
@@ -150,7 +149,8 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
     private volatile boolean destroyed;
 
     // Use an invalid cipherSuite until the handshake is completed
-    // See https://docs.oracle.com/javase/8/docs/api/javax/net/ssl/SSLEngine.html#getSession()
+    // See
+    // https://docs.oracle.com/en/java/javase/11/docs/api/java.base/java/net/Socket.html#setPerformancePreferences(int,int,int)
     private volatile String version;
     private volatile String cipher;
     private volatile String applicationProtocol;
@@ -181,6 +181,8 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
     /**
      * Creates a new instance
      *
+     * @param cleaner                             Used to clean up references to instances before they are garbage
+     *                                                collected
      * @param sslCtx                              an OpenSSL {@code SSL_CTX} object
      * @param fallbackApplicationProtocol         the fallback application protocol
      * @param clientMode                          {@code true} if this is used for clients, {@code false} otherwise
@@ -191,16 +193,17 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
      * @param certificateVerificationDepth        Certificate verification depth
      * @param certificateVerificationOptionalNoCA Skip CA verification in optional mode
      */
-    OpenSSLEngine(long sslCtx, String fallbackApplicationProtocol, boolean clientMode,
+    OpenSSLEngine(Cleaner cleaner, long sslCtx, String fallbackApplicationProtocol, boolean clientMode,
             OpenSSLSessionContext sessionContext, boolean alpn, boolean initialized, int certificateVerificationDepth,
             boolean certificateVerificationOptionalNoCA) {
         if (sslCtx == 0) {
             throw new IllegalArgumentException(sm.getString("engine.noSSLContext"));
         }
-        aprGeneration = Library.getGeneration();
         session = new OpenSSLSession();
-        ssl = SSL.newSSL(sslCtx, !clientMode);
-        networkBIO = SSL.makeNetworkBIO(ssl);
+        long ssl = SSL.newSSL(sslCtx, !clientMode);
+        long networkBIO = SSL.makeNetworkBIO(ssl);
+        state = new OpenSSLState(ssl, networkBIO);
+        cleanable = cleaner.register(this, state);
         this.fallbackApplicationProtocol = fallbackApplicationProtocol;
         this.clientMode = clientMode;
         this.sessionContext = sessionContext;
@@ -221,20 +224,9 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
     public synchronized void shutdown() {
         if (!destroyed) {
             destroyed = true;
+            cleanable.clean();
             // internal errors can cause shutdown without marking the engine closed
             isInboundDone = isOutboundDone = engineClosed = true;
-            if (Library.tryCleanUpLock(aprGeneration)) {
-                try {
-                    if (networkBIO != 0) {
-                        SSL.freeBIO(networkBIO);
-                    }
-                    if (ssl != 0) {
-                        SSL.freeSSL(ssl);
-                    }
-                } finally {
-                    Library.returnCleanUpLock();
-                }
-            }
         }
     }
 
@@ -460,7 +452,7 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
         int pendingNet;
 
         // Check for pending data in the network BIO
-        pendingNet = SSL.pendingWrittenBytesInBIO(networkBIO);
+        pendingNet = SSL.pendingWrittenBytesInBIO(state.networkBIO);
         if (pendingNet > 0) {
             // Do we have enough room in destination to write encrypted data?
             int capacity = dst.remaining();
@@ -470,7 +462,7 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
 
             // Write the pending data from the network BIO into the dst buffer
             try {
-                bytesProduced = readEncryptedData(networkBIO, dst, pendingNet);
+                bytesProduced = readEncryptedData(state.networkBIO, dst, pendingNet);
             } catch (Exception e) {
                 throw new SSLException(e);
             }
@@ -497,13 +489,13 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
 
                 // Write plain text application data to the SSL engine
                 try {
-                    bytesConsumed += writePlaintextData(ssl, src);
+                    bytesConsumed += writePlaintextData(state.ssl, src);
                 } catch (Exception e) {
                     throw new SSLException(e);
                 }
 
                 // Check to see if the engine wrote data into the network BIO
-                pendingNet = SSL.pendingWrittenBytesInBIO(networkBIO);
+                pendingNet = SSL.pendingWrittenBytesInBIO(state.networkBIO);
                 if (pendingNet > 0) {
                     // Do we have enough room in dst to write encrypted data?
                     int capacity = dst.remaining();
@@ -514,7 +506,7 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
 
                     // Write the pending data from the network BIO into the dst buffer
                     try {
-                        bytesProduced += readEncryptedData(networkBIO, dst, pendingNet);
+                        bytesProduced += readEncryptedData(state.networkBIO, dst, pendingNet);
                     } catch (Exception e) {
                         throw new SSLException(e);
                     }
@@ -579,7 +571,7 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
         // Write encrypted data to network BIO
         int written;
         try {
-            written = writeEncryptedData(networkBIO, src);
+            written = writeEncryptedData(state.networkBIO, src);
         } catch (Exception e) {
             throw new SSLException(e);
         }
@@ -618,7 +610,7 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
 
                 int bytesRead;
                 try {
-                    bytesRead = readPlaintextData(ssl, dst);
+                    bytesRead = readPlaintextData(state.ssl, dst);
                 } catch (Exception e) {
                     throw new SSLException(e);
                 }
@@ -645,7 +637,8 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
         }
 
         // Check to see if we received a close_notify message from the peer
-        if (!receivedShutdown && (SSL.getShutdown(ssl) & SSL.SSL_RECEIVED_SHUTDOWN) == SSL.SSL_RECEIVED_SHUTDOWN) {
+        if (!receivedShutdown &&
+                (SSL.getShutdown(state.ssl) & SSL.SSL_RECEIVED_SHUTDOWN) == SSL.SSL_RECEIVED_SHUTDOWN) {
             receivedShutdown = true;
             closeInbound();
         }
@@ -661,24 +654,24 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
         // SSL_pending will return 0 if OpenSSL has not started the current TLS record
         // See https://www.openssl.org/docs/manmaster/man3/SSL_pending.html
         clearLastError();
-        int lastPrimingReadResult = SSL.readFromSSL(ssl, EMPTY_ADDR, 0); // priming read
+        int lastPrimingReadResult = SSL.readFromSSL(state.ssl, EMPTY_ADDR, 0); // priming read
         // check if SSL_read returned <= 0. In this case we need to check the error and see if it was something
         // fatal.
         if (lastPrimingReadResult <= 0) {
             checkLastError();
         }
-        int pendingReadableBytesInSSL = SSL.pendingReadableBytesInSSL(ssl);
+        int pendingReadableBytesInSSL = SSL.pendingReadableBytesInSSL(state.ssl);
 
         // TLS 1.0 needs additional handling
         // TODO Figure out why this is necessary and if a simpler / better
         // solution is available
         if (Constants.SSL_PROTO_TLSv1.equals(version) && lastPrimingReadResult == 0 && pendingReadableBytesInSSL == 0) {
             // Perform another priming read
-            lastPrimingReadResult = SSL.readFromSSL(ssl, EMPTY_ADDR, 0);
+            lastPrimingReadResult = SSL.readFromSSL(state.ssl, EMPTY_ADDR, 0);
             if (lastPrimingReadResult <= 0) {
                 checkLastError();
             }
-            pendingReadableBytesInSSL = SSL.pendingReadableBytesInSSL(ssl);
+            pendingReadableBytesInSSL = SSL.pendingReadableBytesInSSL(state.ssl);
         }
 
         return pendingReadableBytesInSSL;
@@ -724,9 +717,9 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
         engineClosed = true;
 
         if (accepted != Accepted.NOT && !destroyed) {
-            int mode = SSL.getShutdown(ssl);
+            int mode = SSL.getShutdown(state.ssl);
             if ((mode & SSL.SSL_SENT_SHUTDOWN) != SSL.SSL_SENT_SHUTDOWN) {
-                SSL.shutdownSSL(ssl);
+                SSL.shutdownSSL(state.ssl);
             }
         } else {
             // engine closing before initial handshake
@@ -749,7 +742,7 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
         if (destroyed) {
             return new String[0];
         }
-        String[] enabled = SSL.getCiphers(ssl);
+        String[] enabled = SSL.getCiphers(state.ssl);
         if (enabled == null) {
             return new String[0];
         } else {
@@ -798,7 +791,7 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
 
         final String cipherSuiteSpec = buf.toString();
         try {
-            SSL.setCipherSuites(ssl, cipherSuiteSpec);
+            SSL.setCipherSuites(state.ssl, cipherSuiteSpec);
         } catch (Exception e) {
             throw new IllegalStateException(sm.getString("engine.failedCipherSuite", cipherSuiteSpec), e);
         }
@@ -817,7 +810,7 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
         List<String> enabled = new ArrayList<>();
         // Seems like there is no way to explicitly disable SSLv2Hello in OpenSSL so it is always enabled
         enabled.add(Constants.SSL_PROTO_SSLv2Hello);
-        int opts = SSL.getOptions(ssl);
+        int opts = SSL.getOptions(state.ssl);
         if ((opts & SSL.SSL_OP_NO_TLSv1) == 0) {
             enabled.add(Constants.SSL_PROTO_TLSv1);
         }
@@ -870,22 +863,22 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
             }
         }
         // Enable all and then disable what we not want
-        SSL.setOptions(ssl, SSL.SSL_OP_ALL);
+        SSL.setOptions(state.ssl, SSL.SSL_OP_ALL);
 
         if (!sslv2) {
-            SSL.setOptions(ssl, SSL.SSL_OP_NO_SSLv2);
+            SSL.setOptions(state.ssl, SSL.SSL_OP_NO_SSLv2);
         }
         if (!sslv3) {
-            SSL.setOptions(ssl, SSL.SSL_OP_NO_SSLv3);
+            SSL.setOptions(state.ssl, SSL.SSL_OP_NO_SSLv3);
         }
         if (!tlsv1) {
-            SSL.setOptions(ssl, SSL.SSL_OP_NO_TLSv1);
+            SSL.setOptions(state.ssl, SSL.SSL_OP_NO_TLSv1);
         }
         if (!tlsv1_1) {
-            SSL.setOptions(ssl, SSL.SSL_OP_NO_TLSv1_1);
+            SSL.setOptions(state.ssl, SSL.SSL_OP_NO_TLSv1_1);
         }
         if (!tlsv1_2) {
-            SSL.setOptions(ssl, SSL.SSL_OP_NO_TLSv1_2);
+            SSL.setOptions(state.ssl, SSL.SSL_OP_NO_TLSv1_2);
         }
     }
 
@@ -926,14 +919,14 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
     }
 
     private void handshake() throws SSLException {
-        currentHandshake = SSL.getHandshakeCount(ssl);
+        currentHandshake = SSL.getHandshakeCount(state.ssl);
         clearLastError();
-        int code = SSL.doHandshake(ssl);
+        int code = SSL.doHandshake(state.ssl);
         if (code <= 0) {
             checkLastError();
         } else {
             if (alpn) {
-                selectedProtocol = SSL.getAlpnSelected(ssl);
+                selectedProtocol = SSL.getAlpnSelected(state.ssl);
             }
             session.lastAccessedTime = System.currentTimeMillis();
             // if SSL_do_handshake returns > 0 it means the handshake was finished. This means we can update
@@ -945,10 +938,10 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
     private synchronized void renegotiate() throws SSLException {
         clearLastError();
         int code;
-        if (SSL.getVersion(ssl).equals(Constants.SSL_PROTO_TLSv1_3)) {
-            code = SSL.verifyClientPostHandshake(ssl);
+        if (SSL.getVersion(state.ssl).equals(Constants.SSL_PROTO_TLSv1_3)) {
+            code = SSL.verifyClientPostHandshake(state.ssl);
         } else {
-            code = SSL.renegotiate(ssl);
+            code = SSL.renegotiate(state.ssl);
         }
         if (code <= 0) {
             checkLastError();
@@ -956,8 +949,8 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
         handshakeFinished = false;
         peerCerts = null;
         x509PeerCerts = null;
-        currentHandshake = SSL.getHandshakeCount(ssl);
-        int code2 = SSL.doHandshake(ssl);
+        currentHandshake = SSL.getHandshakeCount(state.ssl);
+        int code2 = SSL.doHandshake(state.ssl);
         if (code2 <= 0) {
             checkLastError();
         }
@@ -1021,7 +1014,7 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
         if (!handshakeFinished) {
 
             // There is pending data in the network BIO -- call wrap
-            if (sendHandshakeError || SSL.pendingWrittenBytesInBIO(networkBIO) != 0) {
+            if (sendHandshakeError || SSL.pendingWrittenBytesInBIO(state.networkBIO) != 0) {
                 if (sendHandshakeError) {
                     // After a last wrap, consider it is going to be done
                     sendHandshakeError = false;
@@ -1053,14 +1046,14 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
 
             // No pending data to be sent to the peer
             // Check to see if we have finished handshaking
-            int handshakeCount = SSL.getHandshakeCount(ssl);
-            if (handshakeCount != currentHandshake && SSL.renegotiatePending(ssl) == 0 &&
-                    (SSL.getPostHandshakeAuthInProgress(ssl) == 0)) {
+            int handshakeCount = SSL.getHandshakeCount(state.ssl);
+            if (handshakeCount != currentHandshake && SSL.renegotiatePending(state.ssl) == 0 &&
+                    (SSL.getPostHandshakeAuthInProgress(state.ssl) == 0)) {
                 if (alpn) {
-                    selectedProtocol = SSL.getAlpnSelected(ssl);
+                    selectedProtocol = SSL.getAlpnSelected(state.ssl);
                 }
                 session.lastAccessedTime = System.currentTimeMillis();
-                version = SSL.getVersion(ssl);
+                version = SSL.getVersion(state.ssl);
                 handshakeFinished = true;
                 return SSLEngineResult.HandshakeStatus.FINISHED;
             }
@@ -1073,7 +1066,7 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
 
         // Check if we are in the shutdown phase
         if (engineClosed) {
-            if (SSL.pendingWrittenBytesInBIO(networkBIO) != 0) {
+            if (SSL.pendingWrittenBytesInBIO(state.networkBIO) != 0) {
                 // Waiting to send the close_notify message
                 return SSLEngineResult.HandshakeStatus.NEED_WRAP;
             }
@@ -1130,13 +1123,13 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
             }
             switch (mode) {
                 case NONE:
-                    SSL.setVerify(ssl, SSL.SSL_CVERIFY_NONE, certificateVerificationDepth);
+                    SSL.setVerify(state.ssl, SSL.SSL_CVERIFY_NONE, certificateVerificationDepth);
                     break;
                 case REQUIRE:
-                    SSL.setVerify(ssl, SSL.SSL_CVERIFY_REQUIRE, certificateVerificationDepth);
+                    SSL.setVerify(state.ssl, SSL.SSL_CVERIFY_REQUIRE, certificateVerificationDepth);
                     break;
                 case OPTIONAL:
-                    SSL.setVerify(ssl, certificateVerificationOptionalNoCA ? SSL.SSL_CVERIFY_OPTIONAL_NO_CA :
+                    SSL.setVerify(state.ssl, certificateVerificationOptionalNoCA ? SSL.SSL_CVERIFY_OPTIONAL_NO_CA :
                             SSL.SSL_CVERIFY_OPTIONAL, certificateVerificationDepth);
                     break;
             }
@@ -1157,12 +1150,6 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
         return true;
     }
 
-    @Override
-    protected void finalize() throws Throwable {
-        super.finalize();
-        // Call shutdown as the user may have created the OpenSslEngine and not used it at all.
-        shutdown();
-    }
 
     private class OpenSSLSession implements SSLSession {
 
@@ -1177,7 +1164,7 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
             byte[] id = null;
             synchronized (OpenSSLEngine.this) {
                 if (!destroyed) {
-                    id = SSL.getSessionId(ssl);
+                    id = SSL.getSessionId(state.ssl);
                 }
             }
 
@@ -1195,7 +1182,7 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
             long creationTime = 0;
             synchronized (OpenSSLEngine.this) {
                 if (!destroyed) {
-                    creationTime = SSL.getTime(ssl);
+                    creationTime = SSL.getTime(state.ssl);
                 }
             }
             return creationTime * 1000L;
@@ -1283,17 +1270,17 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
                 byte[] clientCert;
                 byte[][] chain;
                 synchronized (OpenSSLEngine.this) {
-                    if (destroyed || SSL.isInInit(ssl) != 0) {
+                    if (destroyed || SSL.isInInit(state.ssl) != 0) {
                         throw new SSLPeerUnverifiedException(sm.getString("engine.unverifiedPeer"));
                     }
-                    chain = SSL.getPeerCertChain(ssl);
+                    chain = SSL.getPeerCertChain(state.ssl);
                     if (!clientMode) {
                         // if used on the server side SSL_get_peer_cert_chain(...) will not include the remote peer
                         // certificate.
                         // We use SSL_get_peer_certificate to get it in this case and add it to our array later.
                         //
                         // See https://www.openssl.org/docs/ssl/SSL_get_peer_cert_chain.html
-                        clientCert = SSL.getPeerCertificate(ssl);
+                        clientCert = SSL.getPeerCertificate(state.ssl);
                     } else {
                         clientCert = null;
                     }
@@ -1340,10 +1327,10 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
             if (c == null) {
                 byte[][] chain;
                 synchronized (OpenSSLEngine.this) {
-                    if (destroyed || SSL.isInInit(ssl) != 0) {
+                    if (destroyed || SSL.isInInit(state.ssl) != 0) {
                         throw new SSLPeerUnverifiedException(sm.getString("engine.unverifiedPeer"));
                     }
-                    chain = SSL.getPeerCertChain(ssl);
+                    chain = SSL.getPeerCertChain(state.ssl);
                 }
                 if (chain == null) {
                     throw new SSLPeerUnverifiedException(sm.getString("engine.unverifiedPeer"));
@@ -1380,7 +1367,7 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
         }
 
         private Principal principal(Certificate[] certs) {
-            return ((java.security.cert.X509Certificate) certs[0]).getSubjectX500Principal();
+            return ((java.security.cert.X509Certificate) certs[0]).getIssuerX500Principal();
         }
 
         @Override
@@ -1394,7 +1381,7 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
                     if (destroyed) {
                         return INVALID_CIPHER;
                     }
-                    ciphers = SSL.getCipherForSSL(ssl);
+                    ciphers = SSL.getCipherForSSL(state.ssl);
                 }
                 String c = OpenSSLCipherConfigurationParser.openSSLToJsse(ciphers);
                 if (c != null) {
@@ -1418,7 +1405,7 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
             String version = null;
             synchronized (OpenSSLEngine.this) {
                 if (!destroyed) {
-                    version = SSL.getVersion(ssl);
+                    version = SSL.getVersion(state.ssl);
                 }
             }
             if (applicationProtocol.isEmpty()) {
@@ -1451,4 +1438,26 @@ public final class OpenSSLEngine extends SSLEngine implements SSLUtil.ProtocolIn
         }
 
     }
+
+
+    private static class OpenSSLState implements Runnable {
+
+        private final long ssl;
+        private final long networkBIO;
+
+        private OpenSSLState(long ssl, long networkBIO) {
+            this.ssl = ssl;
+            this.networkBIO = networkBIO;
+        }
+
+        @Override
+        public void run() {
+            if (networkBIO != 0) {
+                SSL.freeBIO(networkBIO);
+            }
+            if (ssl != 0) {
+                SSL.freeSSL(ssl);
+            }
+        }
+    }
 }
