@@ -160,8 +160,6 @@ public class TestRegistration extends TomcatBaseTest {
         String protocol = tomcat.getConnector().getProtocolHandlerClassName();
         if (protocol.indexOf("Nio2") > 0) {
             protocol = "nio2";
-        } else if (protocol.indexOf("Apr") > 0) {
-            protocol = "apr";
         } else {
             protocol = "nio";
         }
